/* -*- C++ -*-
 * src/test/graph_adaptor_test.cc - Part of LEMON, a generic C++ optimization library
 *
 * Copyright (C) 2005 Egervary Jeno Kombinatorikus Optimalizalasi Kutatocsoport
 * (Egervary Research Group on Combinatorial Optimization, EGRES).
 *
 * Permission to use, modify and distribute this software is granted
 * provided that this copyright notice appears in all copies. For
 * precise terms see the accompanying LICENSE file.
 *
 * This software is provided "AS IS" with no warranty of any kind,
 * express or implied, and with no claim as to its suitability for any
 * purpose.
 *
 */

#include<iostream>
#include<lemon/concept_check.h>

#include<lemon/smart_graph.h>
#include<lemon/concept/graph.h>
#include<lemon/concept/undir_graph.h>

#include<lemon/list_graph.h>
#include<lemon/full_graph.h>
#include<lemon/graph_adaptor.h>

#include"test/test_tools.h"
#include"test/graph_test.h"

/**
\file
This test makes consistency checks of graph adaptors.

\todo More extensive tests are needed 
*/

using namespace lemon;
using namespace lemon::concept;



int main() 
{
  {
    typedef StaticGraph Graph;
    checkConcept<StaticGraph, GraphAdaptor<Graph> >();

    checkConcept<StaticGraph, RevGraphAdaptor<Graph> >();

    checkConcept<StaticGraph, SubGraphAdaptor<Graph, 
      Graph::NodeMap<bool> , Graph::EdgeMap<bool> > >();
    checkConcept<StaticGraph, NodeSubGraphAdaptor<Graph, 
      Graph::NodeMap<bool> > >();
    checkConcept<StaticGraph, EdgeSubGraphAdaptor<Graph, 
      Graph::EdgeMap<bool> > >();
    
    checkConcept<StaticGraph, SubBidirGraphAdaptor<Graph, 
      Graph::EdgeMap<bool>, Graph::EdgeMap<bool> > >();
    //    checkConcept<StaticGraph, BidirGraph<Graph> >();
    checkConcept<StaticGraph, ResGraphAdaptor<Graph, int, 
      Graph::EdgeMap<int>, Graph::EdgeMap<int> > >();

    checkConcept<StaticGraph, ErasingFirstGraphAdaptor<Graph, 
      Graph::NodeMap<Graph::Edge> > >(); 

    /// \bug why does not compile with StaticGraph
    checkConcept<BaseIterableUndirGraphConcept, UndirGraphAdaptor<ListGraph> >();
    checkConcept<IterableUndirGraphConcept, UndirGraphAdaptor<ListGraph> >();
    checkConcept<MappableUndirGraphConcept, UndirGraphAdaptor<ListGraph> >();
  }
  std::cout << __FILE__ ": All tests passed.\n";

  return 0;
}
